/*******************************************************************************
***  This file is part of QtArgs.                                            ***
***                                                                          ***
***  Copyright (C) 2011, 2012, 2014                                          ***
***  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                    ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify    ***
***  it under the terms of the GNU Lesser General Public License as          ***
***  published by the Free Software Foundation, either version 3 of the      ***
***  License, or (at your option) any later version.                         ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public        ***
***  License along with this program. If not, see                            ***
***  <http://www.gnu.org/licenses/>.                                         ***
*******************************************************************************/
#include "qargs.h"

#include <QChar>
#include <QString>
#include <QCoreApplication>
#include <QFileInfo>
#include <QStringBuilder>
#include <QDebug>
#include <QList>

#include <iostream>
using std::cout;
using std::endl;
#include <stdexcept>
using std::domain_error;
using std::runtime_error;
using std::logic_error;


#ifdef Q_OS_WIN32
    const QChar& QArgs::ARGUMENT_MARKER('/');
    const QChar& QArgs::SHORT_HELP('?');
    const QChar& QArgs::OTHER_OS_SHORT_HELP('h');
#else
    const QChar& QArgs::ARGUMENT_MARKER('-');
    const QChar& QArgs::SHORT_HELP('h');
    const QChar& QArgs::OTHER_OS_SHORT_HELP('?');
#endif
const QChar& QArgs::ARGUMENT_LONG_ASSIGNMENT('=');
const QString& QArgs::LONG_HELP("help");
const qint8 QArgs::MINIMUM_LONG_OPTION(2);

//{ Type Conversions for multiple values
template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, qreal& converted)
{
    bool conversionState;
    converted = value.toDouble(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, qint8& converted)
{
    bool conversionState;
    converted = static_cast<qint8>(value.toShort(&conversionState));
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, qint16& converted)
{
    bool conversionState;
    converted = value.toShort(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, qint32& converted)
{
    bool conversionState;
    converted = value.toLong(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, qint64& converted)
{
    bool conversionState;
    converted = value.toLongLong(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, quint8& converted)
{
    bool conversionState;
    converted = static_cast<quint8>(value.toShort(&conversionState));
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, quint16& converted)
{
    bool conversionState;
    converted = value.toUShort(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, quint32& converted)
{
    bool conversionState;
    converted = value.toULong(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, quint64& converted)
{
    bool conversionState;
    converted = value.toULongLong(&conversionState);
    return conversionState;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, QChar& converted)
{
    bool valid;
    if (value.isEmpty())
	   valid = false;
    else
    {
	   converted = value.at(0);
	   valid = true;
    }
    return valid;
}

template<>
QTARGS_SHARED_EXPORT bool convertToValueType(const QString& value, QString& converted)
{
    converted = value;
    return true;
}
//}

QArgs::QArgs(int argc, const char* argv[], Sensitivity caseing):
    QObject(NULL),
    arguments_(),
    supported_(),
    isCaseSensative_(CASE_SENSATIVE == caseing),
    isLocked_(false),
    areArgumentsVerified_(false)
{
    qDebug() << "QArgs::QArgs(argc, const char*[], Sensitivity)" << argc
		   << *argv << caseing;

    for (int index(0); index < argc; ++index)
	   this->arguments_.append(QString(argv[index]).trimmed());

    Properties help;
    help.shortName = SHORT_HELP;
    help.longName = LONG_HELP;
    help.description = QString();
    help.needsValue = false;
    help.required = false;
    this->supported_.insert(help.shortName, help);
}

QArgs::QArgs(int argc, char* argv[], Sensitivity caseing):
    QObject(NULL),
    arguments_(),
    supported_(),
    isCaseSensative_(CASE_SENSATIVE == caseing),
    isLocked_(false),
    areArgumentsVerified_(false)
{
    qDebug() << "QArgs::QArgs(argc, char*[], Sensitivity)" << argc << *argv
			 << caseing;

    for (int index(0); index < argc; ++index)
	   this->arguments_.append(QString(argv[index]).trimmed());

    Properties help;
    help.shortName = SHORT_HELP;
    help.longName = LONG_HELP;
    help.description = QString();
    help.needsValue = false;
    help.required = false;
    this->supported_.insert(help.shortName, help);
}

const QStringList& QArgs::arguments() const
{
    return this->arguments_;
}

const QChar& QArgs::marker() const
{
    return ARGUMENT_MARKER;
}

qint32 QArgs::count() const
{
    const QString LONG_MARKER(QString("%1%2").arg(ARGUMENT_MARKER).arg(
			 ARGUMENT_MARKER));
    qint32 count(0);

    foreach (QString argument, this->arguments_)
	   if (argument.startsWith(ARGUMENT_MARKER)
			 || argument.startsWith(LONG_MARKER))
		  ++count;
    return count;
}

void QArgs::addSupported(const QChar& shortName, const QString& longName,
	   const QString& description, bool takesValue, bool required)
{
    // Verify short name is valid.
    if ((SHORT_HELP == shortName) || (OTHER_OS_SHORT_HELP == shortName))
	   throw domain_error(tr("Cannot override short "
			 "help argument name, %1.").arg(shortName).toStdString());
    else if (shortName.isNull() || shortName.isSpace())
	   throw logic_error(tr("Option for the short name cannot be "
			 "null or empty.").toStdString());
    else if (this->supported_.contains(shortName))
	   throw logic_error(tr("The short name '%1' was already in the "
			 "supported options list.").arg(shortName).toStdString());

    // Verify long name is valid.
    if (LONG_HELP == longName.trimmed())
	   throw domain_error(tr("Cannot override long "
			 "help argument name, %1.").arg(LONG_HELP).toStdString());
    else if (longName.trimmed().isNull() || (
			  longName.trimmed().count() < MINIMUM_LONG_OPTION))
	   throw logic_error(tr("Option for the long name cannot be "
			 "null, a single character, or empty.").toStdString());
    foreach (Properties supported, this->supported_)
	   if (supported.longName == longName.trimmed())
		  throw logic_error(tr("The long name '%1' was already in the "
						   "supported options list under the '%2' "
						   "short name and cannot be listed under '%3' "
						   "for the short name as well."
						   ).arg(longName.trimmed()
							    ).arg(supported.shortName
									).arg(shortName).toStdString());

    // Check to see if futher options are allowed.
    if (this->isLocked_)
	   throw logic_error(tr("Cannot add futher arguments as the supported "
					    "option list is locked.").toStdString());

    // Add the valid option.
    Properties support;
    support.shortName = shortName;
    support.longName = longName.trimmed();
    support.description = description.trimmed();
    support.needsValue = takesValue;
    support.required = required;
    this->supported_.insert(shortName, support);

    // Guarantee Required Arguments
    if (required && (! hasArgument(shortName)))
	   throw runtime_error(tr("Missing Required Argument: %1(%2) -- %3").arg(
			 shortName).arg(longName.trimmed()).arg(
			 description).toStdString());
}

bool QArgs::hasArgument(const QChar& shortName) const
{
    if ((-1 != shortNameIndex(shortName)) || (-1 != longNameIndex(shortName)))
	   return true;
    else
	   return false;
}

void QArgs::lock()
{
    this->isLocked_ = true;
}

bool QArgs::hasHelp() const
{
    if ((-1 != shortNameIndex(SHORT_HELP)) ||
		  (-1 != longNameIndex(SHORT_HELP)))
	   return true;
    else
	   return false;
}

void QArgs::showHelp() const
{
    QString helpMessage;
    QString fileName((NULL == QCoreApplication::instance()) ?
		  "application" : QCoreApplication::applicationFilePath());

    helpMessage = "usage: " % QFileInfo(fileName).fileName() %
		  " [options]\nOptions:\n";
    foreach (Properties property, this->supported_)
	   if (property.description.isNull())
		  continue;
	   else
		  helpMessage = helpMessage % "  " % ARGUMENT_MARKER %
				property.shortName %
				(property.needsValue ? " <value>\t" : "\t\t" ) %
				"\t" % property.description % "\n";
    cout << helpMessage.toStdString() << endl;
    if (NULL != QCoreApplication::instance())
	   emit help(helpMessage);
}

void QArgs::about() const
{
    QString legalText("QtArgs version ");

    legalText = legalText % LIBRARY_VERSION % "\nCopyright (C) " %
		  COPYRIGHT_YEARS % " CodeGNU Solutions\nQtArgs comes with "
		  "ABSOLUTELY NO WARRANTY.\nThis is free software, and you are " %
		  "welcome to redistribute it\nunder certain conditions; see the "
		  "LICENSE file for details,\nor the Free Software "
		  "Foundation's LGPL.\n";

    cout << legalText.toStdString() << endl;
    if (NULL != QCoreApplication::instance())
	   emit about(legalText);
}

bool QArgs::value(const QChar& shortName, bool defaultValue,
			   BooleanSupport types) const
{
    bool value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);
    QString valueTest;

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   valueTest = this->arguments_.at(shortIndex + 1).toLower();
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   valueTest = this->arguments_.at(longIndex + 1).toLower();
#else
	   valueTest = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toLower();
#endif

    // Generate the boolean value from the supplied option.
    if (((types & QArgs::ONE_OR_ZERO) == QArgs::ONE_OR_ZERO)
		  && (("1" == valueTest) || ("0" == valueTest)))
	   value = (1 == valueTest.toUInt(&converted));
    else if (((types & QArgs::T_OR_F) == QArgs::T_OR_F)
		   && (("t" == valueTest) || ("f" == valueTest)))
	   value = ("t" == valueTest);
    else if (((types & QArgs::TRUE_OR_FALSE) == QArgs::TRUE_OR_FALSE)
		   && (("true" == valueTest) || ("false" == valueTest)))
	   value = ("true" == valueTest);
    else if (((types & QArgs::ON_OR_OFF) == QArgs::ON_OR_OFF)
		   && (("on" == valueTest) || ("off" == valueTest)))
	   value = ("on" == valueTest);
    else if (((types & QArgs::YES_OR_NO) == QArgs::YES_OR_NO)
		   && (("yes" == valueTest) || ("no" == valueTest)))
	   value = ("yes" == valueTest);
    else if (((types & QArgs::Y_OR_N) == QArgs::Y_OR_N)
		   && (("y" == valueTest) || ("n" == valueTest)))
	   value = ("y" == valueTest);
    else
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT qreal QArgs::value(const QChar& shortName, qreal defaultValue) const
{
    qreal value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toDouble(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toDouble(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toDouble(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT qint8 QArgs::value(const QChar& shortName, qint8 defaultValue) const
{
    qint8 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toShort(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toShort(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toShort(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT qint16 QArgs::value(const QChar& shortName, qint16 defaultValue) const
{
    qint16 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toShort(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toShort(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toShort(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT qint32 QArgs::value(const QChar& shortName, qint32 defaultValue) const
{
    qint32 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toInt(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toInt(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toInt(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT qint64 QArgs::value(const QChar& shortName, qint64 defaultValue) const
{
    qint64 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toLongLong(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toLongLong(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toLongLong(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT quint8 QArgs::value(const QChar& shortName, quint8 defaultValue) const
{
    quint8 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toUShort(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toUShort(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toUShort(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT quint16 QArgs::value(const QChar& shortName, quint16 defaultValue) const
{
    quint16 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toUShort(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toUShort(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toUShort(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT quint32 QArgs::value(const QChar& shortName, quint32 defaultValue) const
{
    quint32 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toUInt(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toUInt(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toUInt(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

template<>
QTARGS_SHARED_EXPORT quint64 QArgs::value(const QChar& shortName, quint64 defaultValue) const
{
    quint64 value(defaultValue);
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));
    bool converted(false);

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1).toULongLong(&converted);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1).toULongLong(&converted);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1].toULongLong(&converted);
#endif

    if (! converted)
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

QChar QArgs::value(const QChar& shortName,
	   const QChar& defaultValue) const
{
    QChar value;
    QString valueHolder;
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   valueHolder = this->arguments_.at(shortIndex + 1);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   valueHolder = this->arguments_.at(longIndex + 1);
#else
	   valueHolder = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1];
#endif

    if (valueHolder.isNull() || valueHolder.isEmpty())
	   value = defaultValue;
    else
	   value = valueHolder.at(0);

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

QString QArgs::value(const QChar& shortName,
	   const QString& defaultValue) const
{
    QString value;
    int shortIndex(shortNameIndex(shortName));
    int longIndex(longNameIndex(shortName));

    verifySupport();
    if ((-1 != shortIndex) && ((shortIndex + 1) < this->arguments_.size()))
	   value = this->arguments_.at(shortIndex + 1);
    else if (-1 != longIndex)
#ifdef Q_OS_WIN32
	   value = this->arguments_.at(longIndex + 1);
#else
	   value = this->mArguments.at(longIndex).split(
			 ARGUMENT_LONG_ASSIGNMENT)[1];
#endif

    if (value.isNull() || value.isEmpty())
	   value = defaultValue;

    if (NULL != QCoreApplication::instance())
	   emit argumentValue(shortName, value);
    return value;
}

void QArgs::verifySupport() const
{
    const short SHORT_NAME_SIZE(2);
    const short LONG_NAME_SIZE(3);
    QChar shortName;

    if (! this->areArgumentsVerified_)
    {
	   // Only the help option is supported & user is trying to get a value.
	   if (1 == this->supported_.count())
		  throw logic_error(tr("No arguments listed as "
						   "being supported!").toStdString());

	   foreach (QString argument, this->arguments_)
	   {
		  if ((SHORT_NAME_SIZE == argument.count()) &&
				(argument[0] == ARGUMENT_MARKER) &&
				(argument[1].isLetter()))
			 shortName = argument.at(1);
		  else if ((LONG_NAME_SIZE >= argument.count()) &&
				 (argument[0] == ARGUMENT_MARKER) &&
				 (argument[1] == ARGUMENT_MARKER))
		  {
			 argument = argument.remove(0, 2);
			 argument = argument.split(ARGUMENT_LONG_ASSIGNMENT)[0];
			 shortName = '\0';
			 foreach (Properties property, this->supported_)
				if (property.longName == argument)
				{
				    shortName = property.shortName;
				    break;
				}
		  }
		  else
			 continue;

		  bool supported;
		  if (this->isCaseSensative_)
			 supported = this->supported_.contains(shortName);
		  else
			 supported = (this->supported_.contains(shortName.toUpper())
					    || this->supported_.contains(
						   shortName.toLower()));

		  if (! supported)
			 throw runtime_error(tr("Unrecognized Option '%1'").arg(
								argument).toStdString());
	   }
	   this->areArgumentsVerified_ = true;
    }
}

int QArgs::shortNameIndex(const QChar& name) const
{
    QRegExp indexTest(QString("%1%2").arg(ARGUMENT_MARKER).arg(
					 QRegExp::escape(name)),
				  this->isCaseSensative_ ?
					 Qt::CaseSensitive : Qt::CaseInsensitive);
    int index(this->arguments_.indexOf(indexTest));

#ifdef Q_OS_WIN32
    if (SHORT_HELP == name)
	   index = this->arguments_.indexOf(QString("%1%2").arg(
			 ARGUMENT_MARKER).arg(SHORT_HELP));
#endif
    return index;
}

int QArgs::longNameIndex(const QChar& name) const
{
    int index(-1);

#ifdef Q_OS_WIN32
    // Windows long options form is: /long-index-name value
    index = this->arguments_.indexOf(
			 QRegExp(QString("%1%2").arg(ARGUMENT_MARKER).arg(
					   this->supported_.value(name).longName)));

    if (SHORT_HELP == name)
	   index = this->arguments_.indexOf(QRegExp(QString("%1%2").arg(
			 ARGUMENT_MARKER).arg(supported_.value(name).longName)));
#else
    // Unices long options form is: --long-index-name=value
    index = (this->mArguments.indexOf(
			  QRegExp(QString("%1%2%3%4.*").arg(
					    ARGUMENT_MARKER).arg(ARGUMENT_MARKER).arg(
					    this->mSupported.value(name).longName).arg(
					    ARGUMENT_LONG_ASSIGNMENT))));
#endif

    if (-1 == index) // Test for flag long name argument
	   index = this->arguments_.indexOf(QRegExp(QString("%1%2%3").arg(
			 ARGUMENT_MARKER).arg(ARGUMENT_MARKER).arg(
			 this->supported_.value(name).longName)));
    return index;
}
